import feedparser
import textwrap
# truncate a chunk of html. based on feedparser so I inherit
# its understanding of html, and textwrap for its understanding
# of word breaks. I override 'handle_data' to figure out at
# what point I should stop emitting text, and override unknown*tag
# to make sure tags get closed. only lightly tested.
class _TruncateHTMLProcessor(feedparser._BaseHTMLProcessor):

    def __init__(self, length, encoding='utf-8'):
        self.encoding = encoding
        self.tags = []
        self.recording = True
        self.depth = 0
        self.length = length
        self.wrapper = textwrap.TextWrapper(width=length)
        self.text_so_far = ''
        feedparser._BaseHTMLProcessor.__init__(self, encoding, 'text/html')

    def reset(self):
        feedparser._BaseHTMLProcessor.reset(self)
        self.tags = []
        self.recording = True
        self.depth = 0

    def unknown_starttag(self, tag, attrs):
        self.tags.append(tag)
        if self.recording:
            self.depth = self.depth + 1
            feedparser._BaseHTMLProcessor.unknown_starttag(self, tag, attrs)

    def unknown_endtag(self, tag):
        # ignore omitted end tags
        while len(self.tags) > 0 and self.tags[-1] != tag:
            self.tags.pop()

        # pop this tag
        if len(self.tags) > 0: self.tags.pop()

        # output iff recording or this closes something previously recorded
        if self.recording or len(self.tags) < self.depth:
            self.depth = len(self.tags)
            feedparser._BaseHTMLProcessor.unknown_endtag(self, tag)

    def handle_charref(self, ref):
        # treat entity references as being one character wide
        if self.recording:
            feedparser._BaseHTMLProcessor.handle_charref(self, ref)
            self.text_so_far += 'x'

    def handle_entityref(self, ref):
        # treat entity references as being one character wide
        if self.recording:
            feedparser._BaseHTMLProcessor.handle_entityref(self, ref)
            self.text_so_far += 'x'

    def do_img(self, attrs):
        # replace images with alt tag (if present) or simply <img>
        attrs = dict(attrs)

        if 'alt' in attrs and not attrs['alt']: return

        if 'a' not in self.tags and 'src' in attrs:
            feedparser._BaseHTMLProcessor.handle_data(self,
                '<a href="%s">' % attrs['src'])

        if 'alt' in attrs:
            self.handle_data(attrs['alt'])
        else:
            self.handle_data('&lt;img&gt;')

        if 'a' not in self.tags and 'src' in attrs:
            feedparser._BaseHTMLProcessor.handle_data(self, '</a>')

    def handle_data(self, text):
        if self.recording:
            current = self.text_so_far + text
            lines = self.wrapper.wrap(current)
            if len(lines) > 1:
                self.recording = False
                # truncated text...
                l1 = len(lines[0])
                l2 = len(self.text_so_far)
                if l1 > l2:
                    text = text[0:(l1 - l2)]
                else:
                    text = ''
                text = text + '&hellip;'
            else:
                self.text_so_far = current
            feedparser._BaseHTMLProcessor.handle_data(self, text)

    def handle_comment(self, text):
        if self.recording:
            feedparser._BaseHTMLProcessor.handle_comment(self, text)

    def handle_pi(self, text):
        if self.recording:
            feedparser._BaseHTMLProcessor.handle_pi(self, text)

    def handle_decl(self, text):
        if self.recording:
            feedparser._BaseHTMLProcessor.handle_decl(self, text)
